#include "Consumer.h"

Consumer::Consumer(ProblemDescription *pd)
{
	nNodes = pd->nNodes;
	nSuppliers = pd->nSuppliers;
	demandIntercept = pd->demandIntercept;
	demandSlope = pd->demandSlope;
	totalSupply.resize(nNodes);
	gaps.resize(nNodes);
}


Consumer::~Consumer()
{
}

void Consumer::setInitialPrices(Prices *prices)
{
	for (int i = 0; i < nNodes; i++)
	{
		prices->commodities[i] = 0.5*demandIntercept[i];
		prices->prevComm[i] = 0.5*demandIntercept[i];
	}
	prices->saveComPricesAsPrevious();
	prices->saveComPricesAsPrevious();
}
bool Consumer::setNewCommodityPrices(DecisionVariables *decisions, Prices *prices, double clearingSlack)
{
	bool hasGap = false;
	double gap, prevGap, price, prevPrice, supply, prevSup, demPrice, prevDemPrice, maxPrice;

	prevSupply = totalSupply;
	prevGaps = gaps;
	totalSupply.assign(nNodes,0);
	for (int k = 0; k < nSuppliers; k++)
	{
		for (int i = 0; i < nNodes; i++)
		{
			totalSupply[i] += decisions[k].sales[i];
		}
	}
	if (printGaps) 
		printf("\n\tSupply gaps:");
	for (int i = 0; i < nNodes; i++)
	{
		supply = totalSupply[i];
		prevSup = prevSupply[i];
		price = prices->commodities[i];
		prevPrice = prices->ppcom[i];
		maxPrice = demandIntercept[i];
		demPrice = demandIntercept[i] - demandSlope[i] * supply;
		prevGap = gaps[i]; 
		prevDemPrice = prevPrice + prevGap;
		gap = demPrice - price;
		gaps[i] = gap;

		if (printGaps)
			printf("\t%4.4f", gap);
		if (abs(gaps[i]) > clearingSlack)
		{
			hasGap = true;
			if (price == prevPrice)
				prices->commodities[i] += 0.5*gaps[i];
			else if (price < demPrice)
			{
				//need to increase price
				if (prevPrice < prevDemPrice - clearingSlack)
				{
					//increase more than you did last time
					prices->commodities[i] = fmin(maxPrice, price + 1.5*(price - prevPrice));
				}
				else
				{
					//somewhere between the two prices
					prices->commodities[i] = 0.5 * (price + prevPrice);
				}
			}
			else
			{
				//need to decrease price
				if (prevPrice > prevDemPrice - clearingSlack)
				{
					//decrease more than you did last time
					prices->commodities[i] = fmax(0, price + 1.5*(price - prevPrice));
				}
				else
				{
					//somewhere between the two prices
					prices->commodities[i] = 0.5 * (price + prevPrice);
				}
			}
		}
	}
	return hasGap;
}
