#include "Heuristic.h"


Heuristic::Heuristic()
{

}

//Main loop of the algorithm 
bool Heuristic::runLoop(Prices prices)
{
	if (printRes)
		prices.printPrices();
	t = clock();

	//Solver loop
	while (!reachedGoal && iteration < maxiter)
	{
		if (!hasTSOgap)
		{
			//Getting here means we have solved for current slack values
			printStats(prices, t);

			if (storeValues)
				numpresVector.push_back(std::make_tuple(slack, iteration));
			//if (slack < 1.000001*slackLimit) //todo?
			//{
			//	slack = slackLimit;
			//	reachedGoal = true;
			//	continue;
			//}
			if (commSlack <= targetCommSlack &&
				transSlack <= targetTransSlack &&
				capSlack <= targetCapSlack)
			{
				//slack = slackLimit;
				reachedGoal = true;
				continue;
			}
			commSlack = fmax(commSlack / slack_change_factor, targetCommSlack);
			transSlack = fmax(transSlack / slack_change_factor, targetTransSlack);
			capSlack = fmax(capSlack / slack_change_factor, targetCapSlack);
			reducedSlack++;
			i1 = 0;
			i2tot = 0;
			slack = fmax(slack / slack_change_factor, slackLimit);
			if (reduceStep)
				tso->reduceStep0(1 / slack_change_factor);
		}
		//init
		iteration++;
		if (iteration == 50)
			hasTSOgap = hasTSOgap;
		i1++;
		i2 = 0;
		prices.saveComPricesAsPrevious();
		prices.saveComPricesAsPrevious();

		//solve suppliers problem for current prices
		//Concurrency::parallel_for(size_t(0), size_t(nSuppliers), [&](size_t i)
#pragma omp parallell for
		for (int i = 0; i < nSuppliers; i++)
		{
			suppliers[i].decide(&prices, &dv[i]);
		}//);
		//storage
		if (storeValues)
		{
			//dvVector1.push_back(dv); push after TSO 
			dvVector2[iteration - 1].push_back(dv);
			priceVector2[iteration - 1].push_back(prices);
		}
		//Commodity loop
		while (consumer->setNewCommodityPrices(&dv[0], &prices, commSlack))
		{
			citeration++;
			i2++;
			prices.saveComPricesAsPrevious();

#pragma omp parallell for
			for (int i = 0; i < nSuppliers; i++)
				suppliers[i].decide(&prices, &dv[i]);
			if (storeValues)
			{
				priceVector2[iteration - 1].push_back(prices);
				dvVector2[iteration - 1].push_back(dv);
			}
		}
		i2tot += i2;
		//Adjust for TSO problem
		hasTSOgap = tso->adjustPrices(&dv[0], &prices, capSlack,
			transSlack, transSlack * dual_cap_slack_scale);
		//storage
		prices.saveFlowPricesAsPrevious();
		if (storeValues)
		{
			dvVector1.push_back(dv);
			priceVector1.push_back(prices);
			iterationVector.push_back(i2);
		}
	}
	//Get solving time
	t = clock() - t;
	runDuration = t;
	totalDecideCalls = nSuppliers*(iteration + citeration) + tso->decideCalls;

	// Do the final logging
	finalLog(prices);

	return true;
}

//Final log after a run
void Heuristic::finalLog(Prices prices)
{
	// If the run did not reach its goal, give the bad news.
	if (!reachedGoal)
	{
		// Print the stats if desired
		if (printRes)
		{
			printf("\n\n-----------------------------------------------------\n\n");
			printf("Did not find solution!\n");
			printf("Reached max iteration at slack level %.2E\n\n", slack);
			printf("Info from last slack level:");
			prices.printPrices(reducedSlack);
			printf("\n\tIterations\t%i\t%i\t%2.2f\t%i", i1, i2tot, (double)i2tot / i1, Supplier::nDecideCalls);
			printf("\nTotal time used: %f", ((float)t) / CLOCKS_PER_SEC);
		}
		// Log the run if desired
		if (storeValues)
			writeResultFile(problemDescription.id, true);
	}
	// If the run was a sucess..
	else
	{
		if (printRes)
		{
			//Print results
			printf("\n\n-----------------------------------------------------\n");
			for (int i = 0; i < nSuppliers; i++)
			{
				suppliers[i].printSolution();
			}
			printf("\n\n");
			tso->printTotalFlows();
			printf("\n");
			prices.printPrices();
			printf("\n\n");
			printf("Numerical precision: %.1E\n", slack);
			printf("\nSolution found in %i TSO iterations and a total of %i commodity iterations giving %i runs of suppliers problems",
				iteration, citeration, iteration + citeration);
			printf("\n");
			printf("\nIt took %d clicks (%f seconds).\n", t, ((float)t) / CLOCKS_PER_SEC);
			printf("Total decide calls: %i", suppliers[0].nDecideCalls);
		}
		if (storeValues)
			writeResultFile(problemDescription.id, false);
	}
}

// Stores the result files in the output folder
void Heuristic::writeResultFile(std::string fileId, bool failed)
{
	if (!writeValues)
		return;
	char sep = ';';
	std::string folderstring = "output/" + fileId;
	std::string folderstring0 = "/Alt" + to_string(problemDescription.alternative);
	std::string tempstring = "";
	if (failed)
		tempstring = "Failed_";
	std::string filestring = folderstring + folderstring0 + "/" + tempstring;
	//tempstring.reserve(5 * iteration*iteration*(sizeof(double)));
	std::ofstream file;
	file.precision(10);

	//price outer loop
	if (CreateDirectory(folderstring.c_str(), NULL) ||
		ERROR_ALREADY_EXISTS == GetLastError())
	{
		folderstring = folderstring + folderstring0;
		if (CreateDirectory(folderstring.c_str(), NULL) ||
			ERROR_ALREADY_EXISTS == GetLastError())
		{
			file.open(filestring + "priceoutput1.csv");
		}
		else
		{
			throw("Failed to create directory");
		}
	}
	else
	{
		throw("Failed to create directory");
	}
	//header
	file << "it1" << sep;
	for (int node = 0; node < nNodes; node++)
		file << "com" << node << sep;
	file << sep;
	for (int link = 0; link < nNodes*nNodes; link++)
		file << "trans" << link << sep;
	file << sep;
	for (int link = 0; link < nNodes*nNodes; link++)
		file << "link" << link << sep;
	file << sep;
	for (int link = 0; link < nNodes*nNodes; link++)
		file << "cong" << link << sep;
	file << "\n";
	//values
	for (int i = 0; i < iteration; i++)
	{
		Prices p = priceVector1[i];
		file << i << sep;
		for (int node = 0; node < nNodes; node++)
			file << p.commodities[node] << sep;
		file << sep;
		for (int link = 0; link < nNodes*nNodes; link++)
			file << p.congestion[link] << sep;
		file << sep;
		for (int link = 0; link < nNodes*nNodes; link++)
			file << p.linkP[link] << sep;
		file << sep;
		for (int link = 0; link < nNodes*nNodes; link++)
			file << p.congRent[link] << sep;
		file << "\n";
	}
	//file << tempstring.str();
	file.close();

	//Price inner loops
	file.open(filestring + "priceoutput2.csv");
	//tempstring.str("");
	//file << "Number of nodes: " << nNodes << "\n\n";
	//header
	file << "it1" << sep << "it2" << sep;
	for (int node = 0; node < nNodes; node++)
		file << "com" << node << sep;
	file << sep;
	for (int link = 0; link < nNodes*nNodes; link++)
		file << "trans" << link << sep;
	file << "\n";
	//values
	for (int i = 0; i < iteration; i++)
	{
		auto pv = priceVector2[i];
		if (pv.size() <= 1)
			continue;
		for (int i2 = 0; i2 < pv.size(); i2++)
		{
			file << i << sep << i2 << sep;
			for (int node = 0; node < nNodes; node++)
				file << pv[i2].commodities[node] << sep;
			file << sep;
			for (int link = 0; link < nNodes*nNodes; link++)
				file << pv[i2].congestion[link] << sep;
			file << "\n";
		}
		file << "\n";
	}
	//file << tempstring.str();
	file.close();

	//Decision variables outer loop
	if (storeDV)
	{
		file.open(filestring + "dvoutput1.csv");
		file << "it1" << sep;
		for (int supplier = 0; supplier < nSuppliers; supplier++)
		{
			for (int node = 0; node < nNodes; node++)
				file << supplier << "sales" << node << sep;
			for (int node = 0; node < nNodes; node++)
				file << supplier << "prod" << node << sep;
			for (int link = 0; link < nNodes*nNodes; link++)
				file << supplier << "trans" << link << sep;
			file << sep;
		}
		file << "\n";
		//values
		for (int i = 0; i < iteration; i++)
		{
			DvVector dvv = dvVector1[i];
			file << i << sep;
			for (int supplier = 0; supplier < nSuppliers; supplier++)
			{
				auto dv = dvv[supplier];
				for (int node = 0; node < nNodes; node++)
					file << dv.sales[node] << sep;
				for (int node = 0; node < nNodes; node++)
					file << dv.production[node] << sep;
				for (int link = 0; link < nNodes*nNodes; link++)
					file << dv.flow[link] << sep;
				file << sep;
			}
			file << "\n";
		}
		//file << file.str();
		file.close();

		//Decision variables inner loop
		file.open(filestring + "dvoutput2.csv");

		//header
		file << "it1" << sep << "it2" << sep;
		for (int supplier = 0; supplier < nSuppliers; supplier++)
		{
			for (int node = 0; node < nNodes; node++)
				file << supplier << "sales" << node << sep;
			for (int node = 0; node < nNodes; node++)
				file << supplier << "prod" << node << sep;
			for (int link = 0; link < nNodes*nNodes; link++)
				file << supplier << "trans" << link << sep;
			file << sep;
		}
		file << "\n";
		//values
		for (int i = 0; i < iteration; i++)
		{
			for (int i2 = 0; i2 < dvVector2[i].size(); i2++)
			{
				DvVector dvv = dvVector2[i][i2];
				file << i << sep << i2;
				for (int supplier = 0; supplier < nSuppliers; supplier++)
				{
					auto dv = dvv[supplier];
					for (int node = 0; node < nNodes; node++)
						file << dv.sales[node] << sep;
					for (int node = 0; node < nNodes; node++)
						file << dv.production[node] << sep;
					for (int link = 0; link < nNodes*nNodes; link++)
						file << dv.flow[link] << sep;
					file << sep;
				}
				file << "\n";
			}
			file << "\n";
		}
		//file << tempstring.str();
		file.close();
	}


	//Iterations file
	file.open(filestring + "iterations.csv");
	//tempstring.str("");
	//slack levels and iterations
	file << "slack level" << sep << "iteration\n";
	for (int i = 0; i < numpresVector.size(); i++)
	{
		file << std::get<0>(numpresVector[i]) << sep << std::get<1>(numpresVector[i]) << "\n";
	}
	file << "\n";
	//header
	file << "inner iterations" << sep << "total inner iterations" << sep;
	file << "total outer iterations" << sep << "average inner pr outer" << sep;
	file << "total runs of supplier problem set\n";
	//values
	int totalInner = std::accumulate(iterationVector.begin(), iterationVector.end(), 0);
	file << iterationVector[0] << sep << totalInner << sep;
	file << iteration << sep << (double)totalInner / iteration << sep;
	file << iteration + totalInner << "\n";
	for (int i = 1; i < iteration; i++)
		file << iterationVector[i] << "\n";
	//file << tempstring.str();
	file.close();
}

// LESS IMPORTANT METHODS BELOW

void Heuristic::setUp(ProblemDescription pd)
{
	if (!first)
	{
		delete tso;
		delete consumer;
	}
	first = false;

	problemDescription = pd;
	nNodes = problemDescription.nNodes;
	nSuppliers = problemDescription.nSuppliers;

	suppliers.resize(0);
	
	// TODO: Improve this..
	bool useTabu = false;
	if (useTabu)
		tso = new TabuTSO(&problemDescription, 20);
	else
		tso = new TSO(&problemDescription);

	for (int i = 0; i < nSuppliers; i++)
	{
		//auto s = new Supplier(&problemDescription, i);
		suppliers.push_back(Supplier(&problemDescription, i));
	}

	tso->suppliers = &suppliers[0];

	consumer = new Consumer(&problemDescription);
	
	clearStorage();

	dv.resize(nSuppliers, DecisionVariables(nNodes));
}

void Heuristic::initRun()
{
	hasSupplyGap = true, hasTSOgap = true;
	reachedGoal = false;
	//set commodity slack factor
	commoditySlackI = 0;
	int cnode = 0;
	for (int node = 0; node < nNodes; node++)
	{
		double intercept = problemDescription.demandIntercept[node];
		if (intercept > 0)
		{
			commoditySlackI += intercept * 0.5;
			cnode++;
		}
	}
	commoditySlackI = commoditySlackI / cnode;
	//set transportation and capacity slack factor
	capacitySlackI = 0;
	transSlackI = 0;
	int num_l = 0;
	for (int l = 0; l < nNodes*nNodes; l++)
	{
		double cap = problemDescription.transCap[l];
		if (cap > 0)
		{
			num_l++;
			transSlackI += problemDescription.transCost1[l] + problemDescription.transCost2[l] * cap;
			capacitySlackI += cap;
		}
	}
	transSlackI = transSlackI / num_l;
	if (problemDescription.alternative == 1)
	{
		capacitySlackI = capacitySlackI / num_l;
		targetCapSlack = targetCapSlackAlt1;
	}
	else
		capacitySlackI = 1;
	
	slack = init_slack;
	commSlack = commoditySlackI * slack * comm_slack_scale;
	transSlack = transSlackI * slack;
	capSlack = capacitySlackI * slack * cap_slack_scale;

	iteration = 0;
	citeration = 0, tsoci = 0, i1 = 0, i2tot = 0, i2;
	reducedSlack = 1;
	for (int i = 0; i < nSuppliers; i++)
		suppliers[i].reset(problemDescription.mosek);
	tso->reset();
	clearStorage();
}

void Heuristic::clearStorage()
{
	Prices p(nNodes);
	priceVector1.resize(0, p);
	priceVector2.resize(0, std::vector<Prices>(0, p));
	dvVector1.resize(0);
	dvVector2.resize(0);
	iterationVector.resize(0);

	if (storeValues)
	{
		priceVector1.reserve(maxiter + 1);
		priceVector2.resize(maxiter + 1, std::vector<Prices>(0, p));
		dvVector1.reserve(maxiter + 1);
		dvVector2.resize(maxiter + 1, std::vector<DvVector>(0));
		iterationVector.reserve(maxiter + 1);
	}
}

void Heuristic::printStats(Prices prices, clock_t runStartTime)
{
	if (printRes)
	{
		prices.printPrices(reducedSlack);
		printf("\n\tIterations\t%i\t%i\t%2.2f", i1, i2tot, (double)i2tot / i1);
		printf("\n\tTime: \t%f", (clock() - t) / 1000.0);
	}
}

bool Heuristic::runCurrentSetup()
{
	//init prices
	Prices prices(nNodes);
	tso->setInitialPrices(&prices);
	consumer->setInitialPrices(&prices);
	return runCurrentSetup(prices, _slackLimit);
}

bool Heuristic::runCurrentSetup(Prices& startPrices)
{
	return runCurrentSetup(startPrices, _slackLimit);
}

bool Heuristic::runCurrentSetup(Prices& startPrices, double slackLimit)
{
	//init
	initRun();
	this->slackLimit = slackLimit;

	return runLoop(startPrices);
}

void Heuristic::printRunSummary()
{
	std::string s;
	if (reachedGoal)
		s = "successfully";
	else
		s = "at numerical bound " + std::to_string(slack) + "";
	printf("%s terminated %s after %i decide calls, taking %f seconds\n", problemDescription.id.c_str(),
		//(reachedGoal ? "successfully" : "at numerical bound " + std::to_string(slack)),
		s.c_str(),
		totalDecideCalls, runDuration / 1000.0);
}

void Heuristic::printRunSummary(std::string runId)
{
	std::string s;
	if (reachedGoal)
		s = "successfully";
	else
		s = "at numerical bound " + std::to_string(slack) + "";
	printf("%s terminated %s after %i decide calls, taking %f seconds\n", runId.c_str(),
		//(reachedGoal ? "successfully" : "at numerical bound " + std::to_string(slack)),
		s.c_str(),
		totalDecideCalls, runDuration / 1000.0);
}

std::vector<std::tuple<double, int>> Heuristic::getNumpresVector()
{
	return numpresVector;
}

std::vector<Prices> Heuristic::getPriceHistory()
{
	return priceVector1;
}

std::vector<DvVector> Heuristic::getDvVector()
{
	return dvVector1;
}

Heuristic::~Heuristic()
{

}
