#pragma once

#include "TSO.h"
#include "Consumer.h"
#include "Supplier.h"
#include <ppl.h>
#include <array>
#include <time.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <numeric>
#include <tuple>

using namespace std;

typedef std::vector<DecisionVariables> DvVector;

//class RunSummary
//{
//
//	int outerIterations, innerIterations;
//	clock_t runDuration;
//	
//};

// Class containing the main algorithm
class Heuristic
{
	std::vector<Supplier> suppliers;
	Consumer *consumer;

	void initRun();
	virtual bool runLoop(Prices initP);
	void printStats(Prices prices, clock_t runStartTime);
	void finalLog(Prices prices);

	//Result storage
	std::vector<Prices> priceVector1;
	std::vector<std::vector<Prices>> priceVector2;// , std::vector<Prices>(0));
	std::vector<DvVector> dvVector1;
	std::vector<std::vector<DvVector>> dvVector2;
	std::vector<int> iterationVector;
	std::vector<std::tuple<double, int>> numpresVector;

	//run variables 
	bool hasSupplyGap = true, hasTSOgap = true;
	bool reachedGoal, first = true;
	double slack, commoditySlackI, transSlackI, capacitySlackI;
	int tsoci = 0, i1 = 0, i2tot = 0, i2;

	//Misc variables
	int nNodes, nSuppliers;
	clock_t t;
	DvVector dv;
	


	void clearStorage();

public:
	Heuristic();
	~Heuristic();
	TSO *tso;

	int iteration, citeration = 0, totalDecideCalls;

	// run parameters (most are overwritten to RunSetup values
	//	in the runLoops class)
	bool storeValues = false, storeDV = false,
		writeValues = true, printRes = true, reduceStep;
	double init_slack = 10, cap_slack_scale = 1000,
		dual_cap_slack_scale = 1, comm_slack_scale;
	double commSlack, transSlack, capSlack;
	double targetCommSlack, targetTransSlack, targetCapSlack, targetCapSlackAlt1;
	int reducedSlack = 1;
	double _slackLimit = 1E-10, slackLimit, slack_change_factor;
	int maxiter = 20000;
	std::string run_id;
	ProblemDescription problemDescription;

	clock_t runDuration;

	//run methods
	virtual void setUp(ProblemDescription);
	bool runCurrentSetup();
	bool runCurrentSetup(Prices& startPrices);
	bool runCurrentSetup(Prices& startPrices, double slackLimit);

	//Misc methods
	void writeResultFile(std::string fileId, bool failed);
	void printRunSummary();
	void printRunSummary(std::string runId);
	//ProblemDescription getBasicProblem2();
	//ProblemDescription getBasicProblem4();
	//ProblemDescription getCircleTwistProblem();
	//ProblemDescription getTwistProblem();
	std::vector<std::tuple<double, int>> getNumpresVector();
	std::vector<Prices> getPriceHistory();
	std::vector<DvVector> getDvVector();
	
};

