#include "Network.h"


Network::Network(){};

void Network::load(int nNodes, vector<double> transcap, vector<double> transCost1, 
	vector<double> transCost2, vector<double> demandIntercept)
{
	this->nNodes = nNodes;
	neighbours.resize(nNodes);
	maxPathCost.resize(nNodes);
	for (int node = 0; node < nNodes; node++)
	{
		maxPathCost[node] = demandIntercept[node];
	}
	this->transCap = transcap;
	this->transCost1 = transCost1;
	this->transCost2 = transCost2;
	nlinks = nNodes*nNodes;
	nLinks_a = 0;
	link_for_a.resize(0);
	a_for_link.resize(nlinks, -1);
	//find usable links a
	for (size_t i = 0; i < nNodes; i++)
	{
		for (size_t j = 0; j < nNodes; j++)
		{
			int link = i*nNodes + j;
			if (i != j && transCap[link] > 0)
			{
				neighbours[i].push_back(j);
				link_for_a.push_back(link);
				a_for_link[link] = nLinks_a;
				nLinks_a++;
				if (transCost1[link] <= 0) 
				{
					throw "?";
				}
			}
		}
	}
	auto t = clock();
	findAllPaths();
	t = clock() - t;
	hasPath.resize(nlinks, 0);
	rs_for_link.resize(nlinks, -1);
	minPrices.resize(nlinks);
	for (size_t i = 0; i < nNodes; i++)
	{
		for (size_t j = 0; j < nNodes; j++)
		{
			int link = i*nNodes + j;
			if (i != j && paths[link].size() > 0)
			{
				hasPath[link] = true;
				link_for_rs.push_back(link);
				rs_for_link[link] = nRS;
				nRS++;
				double minprice = maxPathMC;
				for each (Path p in paths[link])
				{
					minprice = fmin(minprice, p.minMC);
				}
				minPrices[link] = minprice;

			}
			else
				hasPath[link] = false;
		}
	}
}

void Network::findAllPaths()
{
	nPaths = 0;
	paths.resize(nNodes*nNodes);
	paths_for_link.resize(nNodes*nNodes);
	paths_for_a.resize(nLinks_a);
	for (size_t org = 0; org < nNodes; org++)
	{
		for (size_t dest = 0; dest < nNodes; dest++)
		{
			if (org != dest)
				findPaths(org, dest);
		}
	}
}


void Network::findPaths(int org, int dest)
{
	vector<int> visited;
	visited.push_back(org);
	bfs(visited, org, dest, 0, maxPathCost[dest]);
}

void Network::bfs(vector<int> &visited, int org, int dest, double cost, double maxCost)
{
	if (visited.size() == 0)
		return;
	int currentNode = visited.back();
	//for each (int node in neighbours[currentNode])
	//{
	//	if (node == dest && cost + transCost1[currentNode*nNodes + dest] < maxCost)
	//		addPath(org, dest, visited);
	//}
	for each (int node in neighbours[visited.back()])
	{
		int link = currentNode*nNodes + node;
		double costLink = transCost1[link];
		if (node == dest && cost + costLink < maxCost)
		{
			addPath(org, dest, visited);
			continue;
		}
		if (find(visited.begin(), visited.end(), node) != visited.end() || 
			cost + costLink >= maxCost)
			continue;
		visited.push_back(node);
		bfs(visited, org, dest, cost + costLink, maxCost);
		visited.pop_back();
	}
}

void Network::addPath(int org, int dest, vector<int> visited)
{
	visited.push_back(dest);
	Path pa;
	pa.load(visited, nPaths, nNodes, transCap,transCost1,transCost2);
	maxPathMC = fmax(maxPathMC, pa.maxMC);
	nPaths++;
	paths[org*nNodes + dest].push_back(pa);
	for each (int link in pa.arcs)
	{
		paths_for_link[link].push_back(pa);
		paths_for_a[a_for_link[link]].push_back(pa);
	}
}

void Path::load(vector<int> &visited, int i, int nNodes, vector<double> &transcap, 
	vector<double> &transCost1, vector<double> &transCost2)
{
	this->i = i;
	nodes = visited;
	maxMC, minMC = 0;
	for (size_t i = 0; i < visited.size() - 1; i++)
	{
		int node = visited[i];
		int link = node*nNodes + visited[i + 1];
		arcs.push_back(link);
		minMC += transCost1[link];
		maxMC += transCost1[link] + transcap[link] * transCost2[link];
	}
}

bool Path::operator==(const Path& rhs)
{
	if (rhs.nodes.size() != nodes.size())
		return false;
	for (size_t i = 0; i < nodes.size(); i++)
	{
		if (nodes[i] != rhs.nodes[i])
			return false;
	}
	return true;
}


Network::~Network()
{
}
