#include "ProblemDescription.h"


ProblemDescription::ProblemDescription(const char* filename)
{
	transModel = alternative > 1;

	ifstream datafile;
	datafile.open(filename);
	string line;
	if (datafile.is_open())
	{
		while (getline(datafile, line))
		{
			// Ignore GAMS comments
			if (std::regex_match(line, std::regex("(\\\*)(.*)")))
				continue;

			// Antall noder i datasettet
			if (std::regex_match(line, std::regex("(.*)(nodes )(.*)")))
			{
				this->nNodes = atoi(line.substr(line.find("*") + 1, line.find_last_of("/") - line.find("*") - 1).c_str());
				continue;
			}
			// Antall produsenter i datasettet
			if (std::regex_match(line, std::regex("(.*)(producers )(.*)")))
			{
				this->nSuppliers = atoi(line.substr(line.find("*") + 1, line.find_last_of("/") - line.find("*") - 1).c_str());
				continue;
			}
			// Demand slope i de ulike markedene
 			if (std::regex_match(line, std::regex("(.*)(slope)(.*)")))
			{
				demandSlope.resize(nNodes);
				for (int i = 0; i < nNodes; i++)
				{
					getline(datafile, line);

					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;
					iss >> sub;
					this->demandSlope[i] = atof(sub.c_str());
				}
				continue;
			}

			// Demand intecept i de ulike market
			if (std::regex_match(line, std::regex("(.*)(intercept)(.*)")))
			{
				demandIntercept.resize(nNodes);
				for (int i = 0; i < nNodes; i++)
				{
					getline(datafile, line);
					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;
					iss >> sub;
					this->demandIntercept[i] = atof(sub.c_str());
				}
				continue;
			}

			// Linear prodkost for hver produsent
			if (std::regex_match(line, std::regex("(.*)(linear supplier cost)(.*)")))
			{
				prodCost1.resize(nNodes*nSuppliers);
				for (int i = 0; i < nSuppliers; i++)
				{
					getline(datafile, line);
					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;
					iss >> sub;

					double currentCost = atof(sub.c_str());
					for (int j = 0; j < nNodes; j++)
						prodCost1[j*nSuppliers + i] = currentCost;
				}
				continue;
			}

			// Quad prodkost for hver produsent
			if (std::regex_match(line, std::regex("(.*)(quad supplier cost)(.*)")))
			{
				prodCost2.resize(nNodes*nSuppliers);
				for (int i = 0; i < nSuppliers; i++)
				{
					getline(datafile, line);

					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;
					iss >> sub;
					
					double currentCost = atof(sub.c_str());
					for (int j = 0; j < nNodes; j++)
						prodCost2[j*nSuppliers + i] = currentCost;
				}
				continue;
			}

			// Linear tsokost for hvert par av noder
			if (std::regex_match(line, std::regex("(.*)(linear tso cost)(.*)")))
			{
				transCost1.resize(nNodes*nNodes);
				getline(datafile, line); // Skip the index line
				// For each node from..
				for (int from = 0; from < nNodes; from++)
				{
					getline(datafile, line);
					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;

					for (int to = 0; to < nNodes; to++)
					{
						iss >> sub;
						transCost1[from*nNodes + to] = atof(sub.c_str());
					}
				}
				continue;
			}

			// Quad tsokost for hvert par av noder
			if (std::regex_match(line, std::regex("(.*)(quad tso cost)(.*)")))
			{
				transCost2.resize(nNodes*nNodes);
				getline(datafile, line); // Skip the index line
				// For each node from..
				for (int from = 0; from < nNodes; from++)
				{
					getline(datafile, line);
					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;

					for (int to = 0; to < nNodes; to++)
					{
						iss >> sub;
						transCost2[from*nNodes + to] = atof(sub.c_str());
					}
				}
				continue;
			}

			if (std::regex_match(line, std::regex("(.*)(production capacity)(.*)")))
			{
				prodCap.resize(nNodes*nSuppliers);
				getline(datafile, line); // Skip the index line
				// For each node from..
				for (int i = 0; i < nNodes; i++)
				{
					getline(datafile, line);
					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;

					for (int j = 0; j < nSuppliers; j++)
					{
						iss >> sub;
						prodCap[i*nSuppliers + j] = atof(sub.c_str());
					}
				}
				continue;
			}


			if (std::regex_match(line, std::regex("(.*)(transportation capacity)(.*)")))
			{
				transCap.resize(nNodes*nNodes);
				getline(datafile, line); // Skip the index line
				// For each node from..
				for (int from = 0; from < nNodes; from++)
				{
					getline(datafile, line);
					// Split the string
					istringstream iss(line);
					// Void the index column
					string sub;
					iss >> sub;

					for (int to = 0; to < nNodes; to++)
					{
						iss >> sub;
						transCap[from*nNodes + to] = atof(sub.c_str());
					}
				}
				continue;
			}
			
			printf(line.c_str());
			printf("\n");
			//transTariff.resize(nNodes*nNodes);

		}
	}
	datafile.close();

	//network.load(nNodes, transCap,transCost1,transCost2, demandIntercept);
};

ProblemDescription::~ProblemDescription()
{

};

ProblemDescription::ProblemDescription()
{
	transModel = alternative > 1;
}

ProblemDescription::ProblemDescription(const int nNodes, const int nSuppliers)
{ 
	this->nNodes = nNodes;
	this->nSuppliers = nSuppliers;

	demandSlope.resize(nNodes);
	demandIntercept.resize(nNodes);
	prodCost1.resize(nNodes*nSuppliers);
	prodCost2.resize(nNodes*nSuppliers);
	transCost1.resize(nNodes*nNodes);
	transCost2.resize(nNodes*nNodes);
	transCap.resize(nNodes*nNodes);
	prodCap.resize(nNodes*nSuppliers);

	transModel = alternative > 1;
}

bool ProblemDescription::isValidSolution(DecisionVariables decisions, Prices prices, double slack)
{
	//skip supplier checks as these are solved with solver
	
	return true;
}

void ProblemDescription::saveToFile(string id, string folderpath)
{
	this->id = id;
	saveToFile();
}

void ProblemDescription::saveToFile(string folderpath)
{
	string filepath = folderpath + "/" + id + ".qex";
	ofstream oStream(filepath, ios::binary);
	fileHeader_t fHeader;
	fHeader.magicNumber = 0xDEADBEAF;
	fHeader.nNodes = nNodes;
	fHeader.nSuppliers = nSuppliers;
	oStream.write((char*)&fHeader, sizeof(fileHeader_t));
	
	writeVector(demandSlope, oStream);
	writeVector(demandIntercept, oStream);
	writeVector(prodCost1, oStream);
	writeVector(prodCost2, oStream);
	writeVector(transCost1, oStream);
	writeVector(transCost2, oStream);
	writeVector(prodCap, oStream);
	writeVector(transCap, oStream);

	oStream.close();
}

void ProblemDescription::convertGamsToQex(string gamsf)
{
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind;
	string folderPath = gamsf + "/*.INC";
	hFind = FindFirstFile(folderPath.c_str(), &FindFileData);
	if (hFind == INVALID_HANDLE_VALUE)
	{
		throw "lolololol";
	}
	else do{
		cout << FindFileData.cFileName << endl;
		string filen = gamsf + "/" + FindFileData.cFileName;
		ProblemDescription pd(filen.c_str());
		string pid;
		pid.assign(FindFileData.cFileName);
		pid.resize(pid.size() - 4);
		pd.id = pid;
		pd.saveToFile();
	} while (FindNextFile(hFind, &FindFileData));
}

void ProblemDescription::loadFromFile(string idString, string folderpath)
{
	id = idString;
	string filepath = folderpath + "/" + id + ".qex";
	ifstream iStream(filepath, ios::binary);
	if (!iStream)
	{
		cout << "No file";
		throw "could not find file!";
	}
	else {
		fileHeader_t fHeader;
		iStream.read((char*)&fHeader, sizeof(fileHeader_t));
		if (fHeader.magicNumber != 0xDEADBEAF) 
			throw "error in header!";
		nNodes = fHeader.nNodes;
		nSuppliers = fHeader.nSuppliers;
		
		readVector(demandSlope, iStream);
		readVector(demandIntercept, iStream);
		readVector(prodCost1, iStream);
		readVector(prodCost2, iStream);
		readVector(transCost1, iStream);
		readVector(transCost2, iStream);
		readVector(prodCap, iStream);
		readVector(transCap, iStream);

		iStream.close();

		if (transModel)
			network.load(nNodes, transCap, transCost1,transCost2, demandIntercept);


	}
}

void ProblemDescription::activateTransModel()
{
	transModel = true;
	network.load(nNodes, transCap, transCost1, transCost2, demandIntercept);
}

void ProblemDescription::scaleDataSet()
{
	for (size_t i = 0; i < nNodes; i++)
	{
		demandSlope[i] *= 1000;
		demandIntercept[i] *= 1000;
		for (size_t j = 0; j < nNodes; j++)
		{

		}
	}
}

vector<ProblemDescription> ProblemDescription::loadAllProblems()
{
	string folderPath = "data/quickLoad";
	return loadProblemFolder(folderPath);
}

vector<ProblemDescription> ProblemDescription::loadProblemFolder(string folderPath)
{
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind;
	vector<ProblemDescription> vec;
	string folderPathS = folderPath + "/*.qex";
	hFind = FindFirstFile(folderPathS.c_str(), &FindFileData);
	if (hFind == INVALID_HANDLE_VALUE)	
	{
		throw "lolololol";
	}
	else do{
		cout << FindFileData.cFileName << endl;
		ProblemDescription pd;
		string pid;
		pid.assign(FindFileData.cFileName);
		pid.resize(pid.size() - 4);
		pd.loadFromFile(pid, folderPath);
		vec.push_back(pd);
	} while (FindNextFile(hFind, &FindFileData));

	return vec;
}

void ProblemDescription::updateDatasets()
{
	// Loop trough all datasets in data/gms/datasets, and run them
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind;
	string folderPath = "data/gms/datasets/*.INC";
	hFind = FindFirstFile(folderPath.c_str(), &FindFileData);
	if (hFind == INVALID_HANDLE_VALUE)
		throw "Error in solving and parsing the datasets";
	else do {
		
		string temp = FindFileData.cFileName;
		ProblemDescription::updateDataset(FindFileData.cFileName);

	} while (FindNextFile(hFind, &FindFileData));
}

void ProblemDescription::updateDataset(string name)
{
	// Simple error-checking -- look for the include file

	// Check if the last part of the string is .INC and remove it if it is
	if (name.length() > 4 && name.substr(name.length() - 4) == ".INC")
		name = name.substr(0, name.length() - 4);

	// Perform the system call
	system(("gams model.gms u1=\"" + name + "\"").c_str());

	// If everything worked, the result file is written to data/gms/results. Parse in those prices, and quickSave them
	Prices eqPrices = Prices(name);
	DecisionVariables eqDV = DecisionVariables(name);


	ofstream oStream("data/gms/results/quickLoad/" + name + "_solution.qex" ,ios::binary);
	fileHeader_t fHeader;
	fHeader.magicNumber = 0xDEADBEAF;
	fHeader.nNodes = eqPrices.nNodes;
	// Todo:Consider suppliers?
	oStream.write((char*)&fHeader, sizeof(fileHeader_t));

	writeVector(eqPrices.commodities, oStream);
	writeVector(eqPrices.congestion, oStream);
	writeVector(eqDV.production, oStream);
	writeVector(eqDV.sales, oStream);
	writeVector(eqDV.flow, oStream);
	writeVector(eqDV.duals, oStream);

	oStream.close();

	// That should be it -- return and move to the next problem

}

void ProblemDescription::writeVector(doubleVector &vec, ofstream &s)
{
	unsigned int len = vec.size();
	s.write((char*)&len, sizeof(len));
	s.write((char*)&vec[0], len*sizeof(vec[0]));
}

void ProblemDescription::readVector(doubleVector& vec, istream &s)
{
	unsigned int len = 0;
	s.read((char*)&len, sizeof(len));
	vec.resize(len);
	if (len == 0) return;
	s.read((char *)&vec[0], len*sizeof(vec[0]));
	/*for (size_t i = 0; i < vec.size(); i++)
	{
		vec[i] *= 10E3;
	}*/
}

SupplierDescription::SupplierDescription(ProblemDescription* pd, int n){
	supplierNumber = n;
	problemDescription = pd;
	nNodes = pd->nNodes;
	nSuppliers = pd->nSuppliers;
	demandSlope = pd->demandSlope;
	prodCost1.resize(nNodes);
	prodCost2.resize(nNodes);
	prodCap.resize(nNodes);
	for (int i = 0; i < nNodes; i++)
	{
		prodCost1[i] = pd->prodCost1[i*nSuppliers + n];
		prodCost2[i] = pd->prodCost2[i*nSuppliers + n];
		prodCap[i] = pd->prodCap[i*nSuppliers + n];
	}
	transCap = pd->transCap;
}

Prices::Prices(int nNodes)
{
	this->nNodes = nNodes;
	commodities.resize(nNodes);
	congestion.resize(nNodes*nNodes);
	prevComm.resize(nNodes);
	prevcon.resize(nNodes*nNodes);
	ppcon.resize(nNodes*nNodes);
	congRent.resize(nNodes*nNodes);
	linkP.resize(nNodes*nNodes);
}

Prices::Prices() : Prices(0) {} // I'm not really sure if this is neeeded.. - M

void Prices::saveFlowPricesAsPrevious()
{
	ppcon = prevcon;
	prevcon = congestion;
}

void Prices::saveComPricesAsPrevious()
{
	ppcom = prevComm;
	prevComm = commodities;
}

void Prices::printPrices()
{
	printf("\nPrices:\tCommodities:");
	for (int i = 0; i < nNodes; i++)
	{
		printf("\t%4.f", commodities[i]);
	}
	printf("\n\tFlows:\t");
	for (int i = 0; i < nNodes; i++)
	{
		for (int j = 0; j < nNodes; j++)
		{
			printf("\t%4.f ", congestion[i*nNodes + j]);
		}
		if (i < nNodes - 1)
			("\n\t\t");
	}
}

void Prices::printPrices(int n)
{
	printf("\n\n%i:\tCommodities:", n);
	for (int i = 0; i < nNodes; i++)
	{
		printf("\t%4.4f", commodities[i]);
	}
	printf("\n\tFlows:\t");
	for (int i = 0; i < nNodes; i++)
	{
		for (int j = 0; j < nNodes; j++)
		{
			printf("\t%4.4f ", congestion[i*nNodes + j]);
		}
		if (i < nNodes - 1)
			("\n\t\t");
	}
}

Prices Prices::loadFromFile(string ids)
{
	//not complete!
	Prices p();
	string filepath = "./data/prices/" + ids + ".price";
	ifstream datafile;
	datafile.open(filepath);
	string line;
	if (!datafile.is_open())
	{
		throw "file not open";
	}
	
	getline(datafile, line);
	int nNodes = atoi(line.c_str());
	Prices prices(nNodes);
	getline(datafile, line);
	for (size_t i = 0; i < nNodes; i++)
	{
		getline(datafile, line);
		prices.commodities[i] = atof(line.c_str());
	}
	getline(datafile, line);
	for (size_t i = 0; i < nNodes*nNodes; i++)
	{
		getline(datafile, line);
		prices.congestion[i] = atof(line.c_str());
	}
	datafile.close();
}

Prices::Prices(string name)
{
	ifstream datafile;
	datafile.open("data/gms/results/" + name + ".csv");
	string line;
	if (datafile.is_open())
	{
		while (getline(datafile, line))
		{
			// Antall produsenter i datasettet
			if (std::regex_match(line, std::regex("(.*)(Nodes)(.*)")))
			{
				this->nNodes = MasterUtility::parseIntDelimited(line, ',', 1);
			}
			else if (std::regex_match(line, std::regex("(.*)(Price)(.*)")))
			{
				this->commodities.resize(nNodes);
				// Assume all the nodes are there and in order -- also possible to add on index
				for (int node = 0; node < nNodes; node++)
				{
					getline(datafile, line);
					// The price is the second element in the csv-line
					this->commodities[node] = MasterUtility::parseDoubleDelimited(line, ',', 1);
				}
			}
			else if (std::regex_match(line, std::regex("(.*)(Tarif)(.*)")))
			{
				// Assume all the links are listed in order
				this->congestion.resize(nNodes*nNodes);
				for (int fromNode = 0; fromNode < nNodes; fromNode++)
				{
					for (int toNode = 0; toNode < nNodes; toNode++)
					{
						getline(datafile, line);
						congestion[fromNode*nNodes + toNode] = MasterUtility::parseDoubleDelimited(line, ',', 2);
					}
				}

			}

		}
	}
	
	datafile.close();

}

double Prices::compareSum(Prices other)
{
	// Return a big number 
	if (this->nNodes != other.nNodes)
		return DBL_MAX;
	double diff = 0;
	for (int fromNode = 0; fromNode < nNodes; fromNode++)
	{
		diff += this->commodities[fromNode] - other.commodities[fromNode];
		for (int toNode = 0; toNode < nNodes; toNode++)
		{
			diff += this->congestion[fromNode*nNodes + toNode] - other.congestion[fromNode*nNodes + toNode];
		}
	}
	return diff;

}

doubleVector Prices::compareComPrice(Prices other)
{
	if (this->nNodes != other.nNodes)
		return {};
	doubleVector diffVec;
	diffVec.resize(this->nNodes);
	for (int node = 0; node < nNodes; node++)
		diffVec[node] = this->commodities[node] - other.commodities[node];
	return diffVec;
}

doubleVector Prices::compareFlowPrice(Prices other)
{
	if (this->nNodes != other.nNodes)
		return{};
	doubleVector diffVec;
	diffVec.resize(nNodes*nNodes);
	for (int link = 0; link < nNodes*nNodes; link++)
		diffVec[link] = this->congestion[link / nNodes + link%nNodes];
	return diffVec;
}

DecisionVariables::DecisionVariables(int nNodes)
{
	this->nNodes = nNodes;
	production.resize(nNodes,0);
	sales.resize(nNodes,0);
	flow.resize(nNodes*nNodes,0);
	duals.resize(nNodes*(2 + nNodes));
	//tsoflows.resize(nNodes*nNodes);
}

DecisionVariables::DecisionVariables(string name)
{
	ifstream datafile;
	datafile.open("data/gms/results/" + name + ".csv");
	string line;
	if (datafile.is_open())
	{
		while (getline(datafile, line))
		{
			if (std::regex_match(line, std::regex("(.*)(Nodes)(.*)")))
			{
				this->nNodes = MasterUtility::parseIntDelimited(line, ',', 1);
				production.resize(nNodes, 0);
				sales.resize(nNodes, 0);
				flow.resize(nNodes*nNodes, 0);
				duals.resize(nNodes*(2 + nNodes));
			}
			else if (std::regex_match(line, std::regex("(.*)(Production)(.*)")))
			{
				for (int node = 0; node < nNodes; node++)
				{
					getline(datafile, line);
					this->production[node] = MasterUtility::parseDoubleDelimited(line, ',', 2);
					this->sales[node] = MasterUtility::parseDoubleDelimited(line, ',', 3);
				}
			}
			else if (std::regex_match(line, std::regex("(.*)(Total flow)(.*)")))
			{
				for (int fromNode = 0; fromNode < nNodes; fromNode++)
				{
					for (int toNode = 0; toNode < nNodes; toNode++)
					{
						getline(datafile, line);
						this->flow[fromNode*nNodes + toNode] = MasterUtility::parseDoubleDelimited(line, ',', 3);
					}
				}
			}

			// TODO: Duals, both in the gams file and here
		}
	}

	datafile.close();

}

DecisionVariables::DecisionVariables(){};

string DecisionVariables::getDiff(DecisionVariables other)
{
	return getDiff(other, false);
}

string DecisionVariables::getDiff(DecisionVariables other, bool ignoreEqual)
{
	// Return a string presenting the difference between this set of dv's and the other. Primary use is to compare gams solution to heuristic solution. If ignoreEqual is true, variables with equal values are left out. The slack below defines equality
	double slack = 0.01;

	// Error checking
	if (nNodes != other.nNodes) 
		return "Different number of nodes. This has " + to_string(nNodes) + " and other has " + to_string(other.nNodes);
	

	// Handle production first
	string res = "------------ PRODUCTION ------------\n";
	for (int producer = 0; producer < nNodes; producer++)
	{
		if (!ignoreEqual || (production[producer] - other.production[producer] < slack))
			res += "Node " + to_string(producer) + ": " + to_string(production[producer] - other.production[producer]) + "\t";
		if (producer > 0 && producer % 5 == 0)
			res += "\n";
	}

	// Deal with sales
	res += "------------   SALES   -------------\n";
	for (int node = 0; node < nNodes; node++)
	{

	}

	return res;
}

void DecisionVariables::print(int nNodes)
{
	printf("\n\tSales:\t");
	for (int i = 0; i < nNodes; i++)
	{
		printf("\t%4.4f ", sales[i]);
	}
	printf("\n\tQuantity:");
	for (int i = 0; i < nNodes; i++)
	{
		printf("\t%4.4f ", production[i]);
	}

	printf("\n\tFlow:\t");
	for (int i = 0; i < nNodes; i++)
	{
		for (int j = 0; j < nNodes; j++)
		{
			printf("\t%4.4f ", flow[i*nNodes + j]);
		}
		printf("\n\t\t");
	}
}

bool ProblemDescription::loadEquilibriumSolution(string name)
{
	// If the file exits, load it
	ifstream iStream("data/gms/results/quickLoad/" + name + ".qex");
	if (!iStream) 
		return false;
	else
	{
		fileHeader_t fHeader;
		iStream.read((char*)&fHeader, sizeof(fileHeader_t));
		if (fHeader.magicNumber != 0xDEADBEAF)
			return false;
		if (fHeader.nNodes != nNodes)
			return false;

		readVector(eqPrices.commodities, iStream);
		readVector(eqPrices.congestion, iStream);
		readVector(eqDV.production, iStream);
		readVector(eqDV.sales, iStream);
		readVector(eqDV.flow, iStream);
		readVector(eqDV.duals, iStream);

		iStream.close();
		return true;
	}
}

bool ProblemDescription::loadEquilibriumSolution() 
{
	return loadEquilibriumSolution(id);
}