#pragma once
#define NOMINMAX
#include <memory>
#include <vector>
#include <set>
#include <iostream>
#include <fstream>
#include <string>
#include <regex>
#include <sstream>
#include <Windows.h>
#include "Network.h"
#include "MasterUtility.h"

using namespace std;

typedef std::vector<double> doubleVector;
typedef struct fileHeader_s
{
	int32_t magicNumber;
	size_t nNodes, nSuppliers;
}fileHeader_t;

class DecisionVariables
{
public:
	doubleVector production, flow, sales, duals;
	//doubleVector tsoflows;
	int nNodes;

	DecisionVariables(int nNodes);
	DecisionVariables(string filePath);
	DecisionVariables();
	void print(int nNodes);

	string getDiff(DecisionVariables other);
	string getDiff(DecisionVariables other, bool ignoreEqual);


};

class Prices
{
public:
	int nNodes;
	doubleVector commodities, congestion, prevComm, ppcom, prevcon, ppcon;
	vector<double> congRent, linkP;

	Prices(int nNodes);
	Prices(string filePath); // Get file prices from a GAMS .csv-file. TODO: faster io. Not an issue with small files.
	Prices();
	double compareSum(Prices other); // Compare two sets of prices and return the sum of elementwise difference. Will return a huge number if the two pricesets are not comparable
	doubleVector compareComPrice(Prices other); // Similar to the above, on vector form
	doubleVector compareFlowPrice(Prices other); // Similar to the above, on vector form
	void setPrevious(Prices &prevPrices);
	void printPrices();
	void printPrices(int);
	void saveFlowPricesAsPrevious();
	void saveComPricesAsPrevious();

	static Prices loadFromFile(string id);
	//void changePrices(doubleVector )
};

class ProblemDescription
{
public:
	// What alternative of the algorithm is compiled is set here!
	int alternative = 1;

	int nNodes, nSuppliers;
	Network network;
	string id;
	doubleVector demandSlope, demandIntercept, prodCost1, prodCost2,
		transCost1, transCost2, prodCap, transCap;
	ProblemDescription();
	ProblemDescription(const char* const filename);
	ProblemDescription(const int nNodes, const int nSuppliers);
	~ProblemDescription();
	bool mosek, transModel; void activateTransModel();
	bool loadSolution = false;

	bool isValidSolution(DecisionVariables, Prices, double);
	void saveToFile(string folderpath = "./data/quickLoad");
	void saveToFile(string id, string folderpath);
	void loadFromFile(string, string folderpath = "./data/quickLoad");
	static void convertGamsToQex(string gamsf);
	static vector<ProblemDescription> loadProblemFolder(string);
	static vector<ProblemDescription> loadAllProblems();

	static void updateDatasets();

	void scaleDataSet();

	static void writeVector(doubleVector&, ofstream&);
	static void readVector(doubleVector&, istream&);

	bool loadTarget(string);

	bool loadEquilibriumSolution(string name);
	bool loadEquilibriumSolution();

	static void updateDataset(string datasetName);
	Prices eqPrices;
	DecisionVariables eqDV;
};

class SupplierDescription
{
public:
	int nNodes, nSuppliers, supplierNumber;
	doubleVector demandSlope, prodCost1, prodCost2,
		prodCap, transCap;
	ProblemDescription* problemDescription;
	SupplierDescription(ProblemDescription* pd, int n);
};



