#include "RunLoops.h"


RunLoops::RunLoops()
{

}

RunLoops::~RunLoops()
{
}

void RunLoops::runSet(vector<ProblemDescription> &set, int n)
{
	loadSet(set);
	runSet(n);
}

void RunLoops::runSet(vector<ProblemDescription> &set)
{
	runSet(set, 1);
}

void RunLoops::runSet(vector<Heuristic> &set, int n)
{
	heurSet = set;
	loadSettings();
	runSet(n);
}

void RunLoops::runSet(vector<Heuristic> &set)
{
	runSet(set, 1);
}

void RunLoops::runSet(Prices prices, int n)
{
	for (int i = 0; i < n; i++)
	{
		//printf("%i\n", i);
		for (size_t j = 0; j < nProb; j++)
		{
			if (prices.nNodes != pdSet[j].nNodes)
				heurSet[j].runCurrentSetup();
			else
				heurSet[j].runCurrentSetup(prices);
			//heuvec[j].writeResultFile("0602");
			if (runSetup.printRes)
				printf("\n-------\n");
			else
				heurSet[j].printRunSummary();

		}
		if (! runSetup.storeValues)
		{
			saveSummary();
		}
		//printf("\n");
		//heu.runCurrentSetup();
		//heu.printRunSummary("Run 1");
		//runTimes.push_back(heu.runDuration);
		//iterations.push_back(heu.iteration);
		//printf("i: %i \n", heu.totalDecideCalls);
		//printf("init_K: %f\n", heu.tso->initPriceK);
		////heu.tso->initPriceK += 0.1;
	}
}

void RunLoops::saveSummary()
{
	char sep = ';';
	std:ofstream file;
	file.open("output/summary.csv", std::ofstream::app);
	file << "Dataset" << sep << "Alternative" << sep << "TSO iterations" << sep << "Commodity iterations" << sep << 
		"TSO supplier calls" << sep << "Calls pr supplier" << sep << "Run duration" <<
		sep << sep << "comSlack" << sep << "transSlack" << sep << "capSlack\n";
	for (int hi = 0; hi < heurSet.size(); hi++)
	{
		Heuristic* h = &heurSet[hi];
		string dataset = h->problemDescription.id;
		int alternative = h->problemDescription.alternative;
		int tsoIterations = h->iteration;
		int commodityIterations = h->citeration;
		int tsoDecideCalls = h->tso->decideCalls;
		int totalSupplerCalls = (tsoIterations + commodityIterations)*h->problemDescription.nSuppliers + tsoDecideCalls;
		double callsPrSupplier = totalSupplerCalls / h->problemDescription.nSuppliers;
		float secondsUsed = ((float)h->runDuration) / CLOCKS_PER_SEC;
		file << dataset << sep << alternative << sep << tsoIterations << sep << commodityIterations << sep <<
			tsoDecideCalls << sep << callsPrSupplier << sep << secondsUsed << 
			sep << sep << h->commSlack << sep << h->transSlack << sep << h->capSlack << "\n";
	}
	file.close();
}

void RunLoops::runSet()
{
	Prices p(0);
	runSet(p);
}

void RunLoops::loadSet(vector<ProblemDescription> &pdSet)
{
	this->pdSet = pdSet;
	nProb = pdSet.size();
	heurSet.resize(nProb);
	for (size_t i = 0; i < nProb; i++)
	{
		heurSet[i].setUp(pdSet[i]);
	}
	loadSettings();
}

void RunLoops::loadSettings(RunSetup &rs)
{
	runSetup = rs;
	loadSettings();
}

void RunLoops::loadSettings()
{
	for (size_t i = 0; i < nProb; i++)
	{
		heurSet[i].init_slack = runSetup.init_slack;
		heurSet[i].cap_slack_scale= runSetup.cap_slack_scale;
		heurSet[i].dual_cap_slack_scale = runSetup.dual_cap_slack_scale;
		heurSet[i].maxiter = runSetup.maxiterations;
		heurSet[i]._slackLimit = runSetup._slackLimit;
		heurSet[i].slackLimit = runSetup._slackLimit;
		heurSet[i].storeValues = runSetup.storeValues;
		heurSet[i].printRes = runSetup.printRes;
		heurSet[i].problemDescription.mosek = runSetup.mosek;
		heurSet[i].slack_change_factor = runSetup.slack_change_factor;
		heurSet[i].comm_slack_scale = runSetup.commodity_slack_scale;
		heurSet[i].reduceStep = runSetup.reduceSteps;
		heurSet[i].targetCapSlack = runSetup.targetCapSlack;
		heurSet[i].targetCapSlackAlt1 = runSetup.targetCapSlackAlt1;
		heurSet[i].targetCommSlack= runSetup.targetCommSlack;
		heurSet[i].targetTransSlack = runSetup.targetTransSlack;
		heurSet[i].storeDV = runSetup.storeDV;

		heurSet[i].tso->initPriceK = runSetup.initPriceK;
		heurSet[i].tso->init_step = runSetup.init_step;
		heurSet[i].tso->step1 = runSetup.step1;
		heurSet[i].tso->step2 = runSetup.step2;
		heurSet[i].tso->init_step_c = runSetup.step_c;
	}
}

void RunLoops::loadFolderSet()
{
	loadSet(ProblemDescription::loadAllProblems());
}

void RunLoops::loadFolderSet(string folder)
{
	loadSet(ProblemDescription::loadProblemFolder(folder));
}

void RunLoops::parameterLogging()
{
	for each (ProblemDescription pd in pdSet)
	{
		// TODO: More intelligent setting of parameters 
		for (double s1 = 0.3; s1 < 0.65; s1 += 0.1) 
		{
			runSetup.step1 = s1;
			runAndLog(pd);
		}
		runSetup.step1 = 0.5;
		for (double s2 = 1.3; s2 < 1.65; s2 += 0.1)
		{
			runSetup.step2 = s2;
			runAndLog(pd);
		}
			

	}

}

void RunLoops::runAndLog(ProblemDescription pd)
{
	// Look for an existing log file for this problem
	std:ofstream file;
	file.open("output/parameter_" + pd.id + ".csv", std::ofstream::app);

	// If the file is empty, write the header
	if (file.tellp() < 1)
		file << "timestamp,problemSet,initPriceK,init_step,step1,step2,_slackLimit,init_slack,slack,iteration\n";

	// Solve the problem!
	Heuristic heur;
	heur.setUp(pd);
	heur.runCurrentSetup();
	// Log the result  -- create one line for each slack level
	std::vector<std::tuple<double, int>> numpresVector = heur.getNumpresVector();
	// Stop the log if the vector is huge.. TODO: Fix/smoothen this
	if (numpresVector.size() > 100)
	{
		file.close();
		return;
	}
	for each (std::tuple<double, int> slackLevel in heur.getNumpresVector())
	{
		file
			<< time(0) << "," << pd.id
			<< "," << runSetup.initPriceK << "," << runSetup.init_step
			<< "," << runSetup.step1 << "," << runSetup.step2 << "," << runSetup._slackLimit
			<< "," << runSetup.init_slack << "," << get<0>(slackLevel) << "," << get<1>(slackLevel) << "\n";
	}
	// Close the file after logging
	file.close();
}

void RunLoops::runAndCompare(ProblemDescription pd)
{
	// Try quickLoading the prices. If not, parse them.
	if (!pd.loadEquilibriumSolution())
	{
		pd.eqPrices = Prices(pd.id);
		pd.eqDV = DecisionVariables(pd.id);
	}

	// Solve the pd USING mosek
	pd.activateTransModel();
	Heuristic heur;
	heur.init_slack = 1000;
	heur.maxiter = 100;
	heur.setUp(pd);
	heur.storeValues = true;
	heur.writeValues = true;
	heur.runCurrentSetup();

	// Compare and print the final result
	Prices finalPrices = heur.getPriceHistory()[heur.getPriceHistory().size() - 1];
	printDiff("commodity prices", MasterUtility::getVectorDifference(pd.eqPrices.commodities, finalPrices.commodities));
	printDiff("transportation prices", MasterUtility::getVectorDifference(pd.eqPrices.congestion, finalPrices.congestion));

	//DvVector foo = heur.getDvVector()[heur.getDvVector().size() - 1];
	//DecisionVariables finalDV = foo[foo.size() - 1];
	//printDiff("production", MasterUtility::getVectorDifference(pd.eqDV.production, finalDV.production));
	//printDiff("sales", MasterUtility::getVectorDifference(pd.eqDV.sales, finalDV.sales));
	//printDiff("flows", MasterUtility::getVectorDifference(pd.eqDV.flow, finalDV.flow));
	//printDiff("duals", MasterUtility::getVectorDifference(pd.eqDV.duals, finalDV.duals));
	
}

void RunLoops::printDiff(string whatChange, std::vector<double> diff)
{
	cout << "\n---------------------------------\nThe " + whatChange + " differ from the target by\n ";
	for each (double d in diff)
		cout << d << "\t";
	cout << "\n";

}