#include "Supplier.h"
#include <string>

int Supplier::nDecideCalls = 0;
int Supplier::nInit = 0;
int Supplier::nHot = 0;
MSKenv_t Supplier::env = NULL;

//Constructor
Supplier::Supplier(ProblemDescription* pd, int suppN)
{
	auto sd = new SupplierDescription(pd, suppN);
	mosek = pd->mosek;
	transModel = pd->transModel;
	description = sd;
	nNodes = description->nNodes;
	hasProduction = false;
	for (int node = 0; node < nNodes; node++)
		if (sd->prodCap [node] > 0)
			hasProduction = true;
	nVariables = nNodes * 2 + nNodes*nNodes;
	if (transModel)
		nVariables = nNodes + nNodes*nNodes;
	nConstraints = nNodes;
	first = true;
	supplierNumber = sd->supplierNumber;
	orgFlowCons = sd->transCap;
	// Old stuff from the qpOases days:
	//nWSR = _nWSR;
	//qProblem = new qpOASES::QProblem(nVariables, nConstraints);
	//options.setToFast();
	//options.boundTolerance = 1E-13;
	//options.terminationTolerance = 1E-10;
	//options.printLevel = qpOASES::PrintLevel(0);
	//qProblem->setOptions(options);

	initMatrixes();
	initMosek();
}

//Initializes to mosek solver for alt 2 and 3
void Supplier::initMosekTransModel()
{
	nProduction = 0;
	productionNodes.resize(0);
	qsub.resize(0);
	qval.resize(0);
	numQnz = 0;
	//quad production cost
	for (size_t node = 0; node < nNodes; node++)
	{
		if (description->prodCap[node] > 0)
		{
			numQnz++;
			nProduction++;
			productionNodes.push_back(node);
			qsub.push_back(node);
			qval.push_back(2 * description->prodCost2[node]);
		}
	}
	//quad sales cost
	for (size_t j = 0; j < nNodes; j++)
	{
		if (description->demandSlope[j])
		{
			for (size_t i = 0; i < nNodes; i++)
			{
				numQnz++;
				int vari = nNodes + i*nNodes + j;
				qsub.push_back(vari);
				qval.push_back(description->demandSlope[j]);
			}
		}
	}

	numAnz = 0;
	aptrb.resize(nConstraints);
	aptre.resize(nConstraints);
	asub.resize(0);
	//A matrix (set in setValMatrixesTransMod)
	for (size_t cons = 0; cons < nConstraints; cons++)
	{
		int rownz = 0;
		for (size_t var = 0; var < nVariables; var++)
		{
			int index = nVariables*cons + var;
			if (A[index] != 0)
			{
				numAnz++;
				asub.push_back(var);
				aval.push_back(A[index]);
				rownz++;
			}
		}
		if (cons == 0)
		{
			aptrb[cons] = 0;
			aptre[cons] = rownz;
		}
		else
		{
			aptrb[cons] = aptre[cons - 1];
			aptre[cons] = aptrb[cons] + rownz;
		}
	}
	//create mosek environment
	if (env == NULL)
		r = MSK_makeenv(&env, NULL);
	else
		r = MSK_RES_OK;
	//create mosek task and insert model
	if (r == MSK_RES_OK)
	{
		r = MSK_maketask(env, nConstraints, nVariables, &task);
		if (r == MSK_RES_OK)
		{
			//r = MSK_linkfunctotaskstream(task, MSK_STREAM_LOG, NULL, mosekPrint);

			if (r == MSK_RES_OK)
				r = MSK_appendcons(task, nConstraints);

			if (r == MSK_RES_OK)
				r = MSK_appendvars(task, nVariables);
			//insert linear production cost
			for (size_t node = 0; node < nNodes; node++)
			{
				if (r == MSK_RES_OK)
					r = MSK_putcj(task, node, description->prodCost1[node]);
			}
			//production bounds
			for (size_t node = 0; node < nNodes; node++)
			{
				MSKboundkeye bkx;
				if (description->prodCap[node] > 0)
					bkx = MSK_BK_RA;
				else
					bkx = MSK_BK_FX;

				if (r == MSK_RES_OK)
					r = MSK_putvarbound(task,
					node,           /* Index of variable.*/
					bkx,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					ub[node]);     /* Numerical value of upper bound.*/
			}
			//sales/flow bounds
			for (size_t j = 0; j < nNodes; j++)
			{
				for (size_t i = 0; i < nNodes; i++)
				{
					MSKboundkeye bkx;
					int linki = i*nNodes + j;
					int vari = nNodes + linki;
					if (i == j || description->problemDescription->network.hasPath[linki])
						bkx = MSK_BK_LO;
					else
						bkx = MSK_BK_FX;

					if (r == MSK_RES_OK)
						r = MSK_putvarbound(task,
						vari,           /* Index of variable.*/
						bkx,      /* Bound key.*/
						0,      /* Numerical value of lower bound.*/
						ub[vari]);     /* Numerical value of upper bound.*/
				}
			}
			//constraints
			for (size_t cons = 0; cons < nConstraints; cons++)
			{
				if (r == MSK_RES_OK)
					r = MSK_putarow(task,
					cons,                 /* Variable (column) index.*/
					aptre[cons] - aptrb[cons], /* Number of non-zeros in column j.*/
					&asub[0] + aptrb[cons],     /* Pointer to row indexes of column j.*/
					&aval[0] + aptrb[cons]);    /* Pointer to Values of column j.*/
				if (r == MSK_RES_OK)
					r = MSK_putconbound(task,
					cons,           /* Index of constraint.*/
					MSK_BK_FX,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					0);     /* Numerical value of upper bound.*/
			}
			//quadratic costs
			if (r == MSK_RES_OK)
				r = MSK_putqobj(task, numQnz, &qsub[0], &qsub[0], &qval[0]);
		}
	}
	if (r != MSK_RES_OK)
		throw "what?";
	MSK_putnaintparam(task, "MSK_IPAR_INTPNT_HOTSTART", MSK_INTPNT_HOTSTART_DUAL);
}

//Initializes the mosek for alt 1
void Supplier::initMosek()
{
	if (transModel)
	{
		initMosekTransModel();
		return;
	}
	nProduction = 0;
	productionNodes.resize(0);
	qsub.resize(nNodes);
	qval.resize(nNodes);
	for (size_t node = 0; node < nNodes; node++)
	{
		qsub[node] = node;
		qval[node] = description->demandSlope[node];
		if (description->prodCap[node] > 0)
		{
			productionNodes.push_back(node);
			nProduction++;
			qsub.push_back(nNodes + node);
			qval.push_back(2 * description->prodCost2[node]);
		}
	}
	numQnz = nNodes + nProduction;
	//Quadratic TSO cost 
	for (size_t link = 0; link < nNodes*nNodes; link++)
	{
		double transcost = description->problemDescription->transCost2[link];
		double transCap = description->transCap[link];
		if (transCap > 0 && transcost > 0)
		{
			qsub.push_back(2 * nNodes + link);
			qval.push_back(quadTransFactor * transcost);
			numQnz++;
		}
	}

	numAnz = 0;
	aptrb.resize(nConstraints);
	aptre.resize(nConstraints);
	asub.resize(0);
	for (size_t cons = 0; cons < nConstraints; cons++)
	{
		int rownz = 0;
		for (size_t var = 0; var < nVariables; var++)
		{
			int index = nVariables*cons + var;
			if (A[index] != 0)
			{
				numAnz++;
				asub.push_back(var);
				aval.push_back(A[index]);
				rownz++;
			}
		}
		if (cons == 0)
		{
			aptrb[cons] = 0;
			aptre[cons] = rownz;
		}
		else
		{
			aptrb[cons] = aptre[cons - 1];
			aptre[cons] = aptrb[cons] + rownz;
		}
	}
	if (env == NULL)
		r = MSK_makeenv(&env, NULL);
	else
		r = MSK_RES_OK;

	if (r == MSK_RES_OK)
	{
		r = MSK_maketask(env, nConstraints, nVariables, &task);
		if (r == MSK_RES_OK)
		{
			//r = MSK_linkfunctotaskstream(task, MSK_STREAM_LOG, NULL, mosekPrint);

			if (r == MSK_RES_OK)
				r = MSK_appendcons(task, nConstraints);

			if (r == MSK_RES_OK)
				r = MSK_appendvars(task, nVariables);

			for (size_t node = 0; node < nNodes; node++)
			{
				if (r == MSK_RES_OK)
					r = MSK_putcj(task, nNodes + node, description->prodCost1[node]);
			}

			for (size_t j = 0; j < nVariables; j++)
			{
				MSKboundkeye bkx;
				if (j < nNodes)
					bkx = MSK_BK_LO; //sales
				else if (j < 2 * nNodes)
				{ // production
					if (description->prodCap[j - nNodes] > 0)
						bkx = MSK_BK_RA;
					else
						bkx = MSK_BK_FX;
				}
				else
				{ //flow
					if (description->transCap[j - 2 * nNodes] > 0)
						bkx = MSK_BK_LO;
					else
						bkx = MSK_BK_FX;
				}

				if (r == MSK_RES_OK)
					r = MSK_putvarbound(task,
					j,           /* Index of variable.*/
					bkx,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					ub[j]);     /* Numerical value of upper bound.*/
			}

			for (size_t cons = 0; cons < nConstraints; cons++)
			{
				if (r == MSK_RES_OK)
					r = MSK_putarow(task,
					cons,                 /* Variable (column) index.*/
					aptre[cons] - aptrb[cons], /* Number of non-zeros in column j.*/
					&asub[0] + aptrb[cons],     /* Pointer to row indexes of column j.*/
					&aval[0] + aptrb[cons]);    /* Pointer to Values of column j.*/
				if (r == MSK_RES_OK)
					r = MSK_putconbound(task,
					cons,           /* Index of constraint.*/
					MSK_BK_FX,      /* Bound key.*/
					0,      /* Numerical value of lower bound.*/
					0);     /* Numerical value of upper bound.*/
			}

			if (r == MSK_RES_OK)
				r = MSK_putqobj(task, numQnz, &qsub[0], &qsub[0], &qval[0]);
		}
	}
	if (r != MSK_RES_OK)
		throw "what?";
	MSK_putnaintparam(task, "MSK_IPAR_INTPNT_HOTSTART", MSK_INTPNT_HOTSTART_DUAL);
}

//Initializes the matrixes used by the solver
void Supplier::initMatrixes()
{
	//allocate 
	H = new double[nVariables*nVariables];
	g = new double[nVariables];
	A = new double[nVariables*nConstraints];
	lbA = new double[nConstraints];
	ubA = new double[nConstraints];
	lb = new double[nVariables];
	ub = new double[nVariables];
	optVal = new double[nVariables];
	//dualOptVal = new double[nVariables*nConstraints];
	dualOptVal.resize(nVariables*nConstraints, -69);
	//optVal.resize(nVariables);
	//dualOptVal.resize(nConstraints);

	//initialize to zero
	for (int i = 0; i < nVariables; i++)
	{
		for (int j = 0; j < nVariables; j++)
		{
			H[i*nVariables + j] = 0;
		}
		g[i] = 0;
		lb[i] = 0;
		ub[i] = 0;
	}
	for (int i = 0; i < nVariables*nConstraints; i++)
	{
		A[i] = 0;
	}
	for (int i = 0; i < nConstraints; i++)
	{
		lbA[i] = 0;
		ubA[i] = 0;
	}
	//set values
	if (transModel)
		setValMatrixesTransMod();
	else
		setValMatrixes();
}

//Solve supplier problem method
void Supplier::decide(const Prices * const prices, DecisionVariables* const decision,
	doubleVector *flowCons)
{
	//skip if we have no production to decide over
	if (!hasProduction)
		return;

	nDecideCalls++;

	if (mosek)
		decideMosek(prices, decision, flowCons);
	else
		throw "qpOASES no longer included";
		//decideQPOases(prices, decision, flowCons);
}

// Run mosek
void Supplier::decideMosek(const Prices * const prices, DecisionVariables* const decision,
	doubleVector *flowCons)
{
	//set prices
	setMosekPrices(prices);
	//set flow constraints
	if (flowCons != nullptr)
		setFlowConstraints(*flowCons);

	MSKrescodee trmcode;

	/* Run optimizer */
	r = MSK_optimizetrm(task, &trmcode);

	/* Print a summary containing information
	about the solution for debugging purposes*/
	//MSK_solutionsummary(task, MSK_STREAM_MSG);

	if (r == MSK_RES_OK)
	{
		MSKsolstae solsta;
		int j;

		MSK_getsolsta(task, MSK_SOL_ITR, &solsta);

		switch (solsta)
		{
		case MSK_SOL_STA_OPTIMAL:
		case MSK_SOL_STA_NEAR_OPTIMAL:
			MSK_getxx(task,
				MSK_SOL_ITR,    /* Request the interior solution. */
				optVal);

			MSK_getsux(task,
				MSK_SOL_ITR, // interior (?) dual
				&dualOptVal[0]);

			break;
		case MSK_SOL_STA_DUAL_INFEAS_CER:
		case MSK_SOL_STA_PRIM_INFEAS_CER:
		case MSK_SOL_STA_NEAR_DUAL_INFEAS_CER:
		case MSK_SOL_STA_NEAR_PRIM_INFEAS_CER:
			printf("Primal or dual infeasibility certificate found.\n");
			break;

		case MSK_SOL_STA_UNKNOWN:
			printf("The status of the solution could not be determined.\n");
			break;
		default:
			printf("Other solution status.");
			break;
		}
	}
	else
	{
		printf("Error while optimizing.\n");
	}

	setMosekDecisions(decision);
	if (flowCons != nullptr)
		resetFlowConstraints();
}

// ---------------- HELPERS AND OUTDATED METHODS BELOW ---------------- 

//Run qpOases (not in use)
//void Supplier::decideQPOases(const Prices * const prices, DecisionVariables* const decision,
//	doubleVector *flowCons)
//{
//	//set prices in g matrix
//	for (int i = 0; i < nNodes; i++)
//	{
//		g[i] = -prices->commodities[i];
//		for (int j = 0; j < nNodes; j++)
//		{
//			g[2 * nNodes + i*nNodes + j] = prices->congestion[i*nNodes + j];
//		}
//	}
//	if (flowCons != nullptr)
//	{
//		setFlowConstraints(*flowCons);
//		changedCap = true;
//	}
//	qpOASES::returnValue res;
//	nWSR = _nWSR;
//	clock_t t = clock();
//	float t2;
//	if (first || changedCap)
//	{
//		//res = qProblem->init(H, g, A, lb, ub, lbA, ubA, nWSR);
//		t2 = (clock() - t) / 1000.0;
//		if (changedCap)
//		{
//			first = true;
//			changedCap = false;
//		}
//		else
//			first = false;
//	}
//	else
//	{
//		res = qProblem->hotstart(g, lb, ub, lbA, ubA, nWSR);
//		t2 = (clock() - t) / 1000.0;
//		nHot++;
//	}
//	t = clock();
//	//check solver status
//	if (res != qpOASES::SUCCESSFUL_RETURN)
//	{
//		//checkMatrices();
//		int nnn = nWSR;
//		nWSR = _nWSR;
//		res = qProblem->init(H, g, A, lb, ub, lbA, ubA, nWSR);
//		t2 = (clock() - t) / 1000.0;
//		nInit++;
//		if (res != qpOASES::SUCCESSFUL_RETURN)
//			throw "Suppliers problem not solved!";
//	}
//	//get optimal values
//	qProblem->getPrimalSolution(optVal);
//	qProblem->getDualSolution(&dualOptVal[0]);
//
//	for (int i = 0; i < nNodes; i++)
//	{
//		decision->sales[i] = optVal[i];
//		decision->production[i] = optVal[nNodes + i];
//		decision->duals[i] = dualOptVal[i];
//		decision->duals[nNodes + i] = dualOptVal[nNodes + i];
//		for (int j = 0; j < nNodes; j++)
//		{
//			decision->flow[i*nNodes + j] = optVal[2 * nNodes + i*nNodes + j];
//			decision->duals[2 * nNodes + i*nNodes + j] = dualOptVal[2 * nNodes + i*nNodes + j];
//		}
//	}
//	if (flowCons != nullptr)
//		resetFlowConstraints();
//
//	//decision->sales = optVal;
//	//decision->production = optVal + nNodes;
//	//decision->flow = optVal + 2 * nNodes;
//}


Supplier::Supplier(){};


void MSKAPI Supplier::mosekPrint(void *handle,
	MSKCONST char str[])
{
	printf("%s", str);
}

void Supplier::setValMatrixesTransMod()
{
	for (size_t i = 0; i < nNodes; i++)
	{
		//Hessian production cost
		H[i * (nVariables + 1)] = 2 * description->prodCost1[i];
		for (size_t j = 0; j < nNodes; j++)
		{
			int link = i*nNodes + j;
			int vari = nNodes + link;
			int Hi = vari * (nVariables + 1);
			//Hessian sales (demand curve slope of destination node)
			H[Hi] = description->demandSlope[j];
			//flow bounds
			if (i == j || description->problemDescription->network.hasPath[link])
				ub[vari] = numeric_limits<double>::max();
			else
				ub[vari] = 0;
		}
		//production limit
		ub[i] = description->prodCap[i];
		//flow conservation (non passing through nodes, just directly from production)
		A[nVariables*i + i] = 1;
		for (size_t j = 0; j < nNodes; j++)
		{
			int vari = nNodes + i*nNodes + j;
			int Ai = i * nVariables + vari;
			if (transModel)
				A[Ai] = -1;
		}
	}
}

void Supplier::setValMatrixes()
{
	for (int i = 0; i < nNodes; i++)
	{
		//Hessian sales (for transmodel this is set in next loop)
		H[nVariables*i + i] = description->demandSlope[i];
		//quadratic production cost parameter
		H[nVariables*(nNodes + i) + nNodes + i] = description->prodCost2[i] * 2;
		// Upper bounds on variables
		ub[i] = std::numeric_limits<double>::max();
		ub[nNodes + i] = description->prodCap[i];
		for (int j = 0; j < nNodes; j++)
		{
			if (description->transCap[nNodes*i + j] == 0)
				ub[2 * nNodes + i * nNodes + j] = 0;
			else if (useTSOcaps)
				ub[2 * nNodes + i * nNodes + j] = description->transCap[i * nNodes + j];
			else
				ub[2 * nNodes + i * nNodes + j] = std::numeric_limits<double>::max();
		}

		// Flow conservation constraint
		A[nVariables*i + i] = 1;
		A[nVariables*i + nNodes + i] = -1;
		for (int j = 0; j < nNodes; j++)
		{
			////set transportation tariff here
			//transTariff[i*nNodes + j] = description->problemDescription->transTariff[i*nNodes + j];
			int vari = 2 * nNodes + i*nNodes + j;
			int Hi = nVariables * (vari + 1);
			if (i != j)
			{
				H[Hi] = quadTransFactor*description->problemDescription->transCost2[i*nNodes + j];
				if (A[nVariables*i + nNodes * 2 + i*nNodes + j] != 0)
					printf("\f\n", A[nVariables*i + nNodes * 2 + i*nNodes + j]);
				if (A[nVariables*i + nNodes * 2 + j*nNodes + i] != 0)
					printf("\f\n", A[nVariables*i + nNodes * 2 + i*nNodes + j]);
				A[nVariables*i + nNodes * 2 + i*nNodes + j] = 1;
				A[nVariables*i + nNodes * 2 + j*nNodes + i] = -1;
			}
		}
		// constant g values (linear cost parameter):
		g[nNodes + i] = description->prodCost1[i];
	}
}

void Supplier::setMosekPrices(const Prices* prices)
{
	if (transModel)
	{
		for (size_t j = 0; j < nNodes; j++)
		{
			for (size_t i = 0; i < nNodes; i++)
			{
				r = MSK_putcj(task, nNodes + i*nNodes + j, 
					prices->congestion[i*nNodes + j] - prices->commodities[j]);
				if (r != MSK_RES_OK)
					throw "what?";
			}
		}
	}
	else
	{
		for (size_t i = 0; i < nNodes; i++)
		{
			double nP = prices->commodities[i];
			double cj;
			r = MSK_getcj(task, i, &cj);
			if (r != MSK_RES_OK)
				throw "what?";
			if (-nP != cj)
				r = MSK_putcj(task, i, -nP);
			if (r != MSK_RES_OK)
				throw "what?";
			for (int j = 0; j < nNodes; j++)
			{
				int vj = 2 * nNodes + i*nNodes + j;
				double lP = prices->congestion[i*nNodes + j];
				r = MSK_getcj(task, vj, &cj);
				if (r != MSK_RES_OK)
					throw "what?";
				if (lP != cj)
					r = MSK_putcj(task, vj, lP);
				if (r != MSK_RES_OK)
					throw "what?";
			}
		}
	}
}

void Supplier::setMosekDecisions(DecisionVariables* const decision)
{
	if (transModel)
	{
		for (size_t j = 0; j < nNodes; j++)
		{
			decision->production[j] = optVal[j];
			decision->duals[j] = dualOptVal[j];
			double sales = 0;
			for (size_t i = 0; i < nNodes; i++)
			{
				double flowVal = optVal[nNodes + i*nNodes + j];
				decision->flow[i*nNodes + j] = flowVal;
				sales += flowVal;
				decision->duals[nNodes + i*nNodes + j] = dualOptVal[nNodes + i*nNodes + j];
			}
			decision->sales[j] = sales;
		}
	}
	else
	{
		for (int i = 0; i < nNodes; i++)
		{
			decision->sales[i] = optVal[i];
			decision->production[i] = optVal[nNodes + i];
			decision->duals[i] = dualOptVal[i];
			decision->duals[nNodes + i] = dualOptVal[nNodes + i];
			for (int j = 0; j < nNodes; j++)
			{
				decision->flow[i*nNodes + j] = optVal[2 * nNodes + i*nNodes + j];
				decision->duals[2 * nNodes + i*nNodes + j] = (-1.0)*dualOptVal[2 * nNodes + i*nNodes + j];
			}
		}
	}
}

void Supplier::setFlowConstraints(doubleVector &cons)
{
	if (mosek)
	{
		MSKboundkeye bkx;
		for (int link = 0; link < nNodes*nNodes; link++)
		{
			if (cons[link] > 0)
				bkx = MSK_BK_RA;
			else
				bkx = MSK_BK_FX;
			r = MSK_putvarbound(task,
				link + 2*nNodes,           /* Index of variable.*/
				bkx,      /* Bound key.*/
				0,      /* Numerical value of lower bound.*/
				cons[link]);     /* Numerical value of upper bound.*/
			if (r != MSK_RES_OK)
				throw "what?";
		}
	}
	else
	{
		for (int link = 0; link < nNodes*nNodes; link++)
		{
			ub[2 * nNodes + link] = cons[link];
		}
	}
}

void Supplier::resetFlowConstraints()
{
	if (mosek)
	{
		MSKboundkeye bkx;
		for (int link = 0; link < nNodes*nNodes; link++)
		{
			if (orgFlowCons[link] > 0)
				bkx = MSK_BK_LO;
			else
				bkx = MSK_BK_FX;
			r = MSK_putvarbound(task,
				link + 2 * nNodes,           /* Index of variable.*/
				bkx,      /* Bound key.*/
				0,      /* Numerical value of lower bound.*/
				orgFlowCons[link]);     /* Numerical value of upper bound.*/
		}
	}
	else
	{
		for (int link = 0; link < nNodes*nNodes; link++)
		{
			if (orgFlowCons[link] == 0)
				ub[2 * nNodes + link] = 0;
			else
				ub[2 * nNodes + link] = std::numeric_limits<double>::max();
		}
	}
}

void Supplier::printSolution()
{
	if (!hasProduction)
		return;
	printf("\nSupplier: %i", supplierNumber);
	printf("\n\tSales:\t");
	for (int i = 0; i < nNodes; i++)
	{
		printf("\t%4.4f ", optVal[i]);
	}
	printf("\n\tQuantity:");
	for (int i = 0; i < nNodes; i++)
	{
		printf("\t%4.4f ", optVal[nNodes + i]);
	}

	printf("\n\tFlow:\t");
	for (int i = 0; i < nNodes; i++)
	{
		for (int j = 0; j < nNodes; j++)
		{
			printf("\t%4.4f ", optVal[2*nNodes + i*nNodes + j]);
		}
		if (i < nNodes-1)
			printf("\n\t\t");
	}
}

void Supplier::checkMatrices()
{
	doubleVector Hv, gv, Av, lbv, ubv, lbAv, ubAv;
	Av.resize(nVariables*nConstraints);

	for (int i = 0; i < nVariables; i++)
	{
		gv.push_back(g[i]);
		lbv.push_back(lb[i]);
		ubv.push_back(ub[i]);
		for (int j = 0; j < nVariables; j++)
		{
			Hv.push_back(H[i*nVariables + j]);
		}
		for (int j = 0; j < nConstraints; j++)
		{
			Av[i*nConstraints + j] = A[i*nConstraints + j];
		}
	}
	for (int  i = 0; i < nConstraints; i++)
	{
		lbAv.push_back(lbA[i]);
		ubAv.push_back(ubA[i]);
	}
	printf("----");
}

void Supplier::reset(bool mosek)
{
	first = true;
	changedCap = false;
	//qProblem->reset();
	this->mosek = mosek;
	if (!mosek)
		mosek = mosek;

	//////delete qProblem;
	//qProblem = new qpOASES::QProblem(nVariables, nConstraints);
	//options.setToFast();
	//options.printLevel = qpOASES::PrintLevel(0);
	//qProblem->setOptions(options);
}

Supplier::~Supplier()
{
	//delete[nVariables*nVariables] H;
	//delete[nVariables] g;
	//delete[nVariables*nConstraints] A;
	//delete[nConstraints] lbA;
	//delete[nConstraints] ubA;
	//delete[nVariables] lb;
	//delete[nVariables] ub;
	//delete[nVariables] optVal;
	//delete[nConstraints] dualOptVal;

}
