#pragma once
#include "ProblemDescription.h"
#include "mosek.h"

class Supplier
{
	int nNodes,nVariables,nProduction,nConstraints, _nWSR = 10000, nWSR, supplierNumber;
	vector<int> productionNodes;
	bool first, useTSOcaps = false, hasProduction, changedCap, 
		mosek = true, transModel;
	double quadTransFactor = 0.00000;
	double *H, *g, *A, *lb, *ub, *lbA, *ubA;
	doubleVector orgFlowCons;
	double *optVal; 
	doubleVector dualOptVal;
	const SupplierDescription* description;
	//qpOASES::QProblem* qProblem;
	//qpOASES::Options options;
	void initMatrixes();
	void setValMatrixes();
	void setValMatrixesTransMod();
	void setFlowConstraints(doubleVector &cons);
	void resetFlowConstraints();
	void initMosek();
	void initMosekTransModel();

	void setMosekPrices(const Prices*); 
	void setMosekDecisions(DecisionVariables* const);

	//mosek variables
	int numAnz, numQnz, numVar, numPath;
	doubleVector c, blc, buc, blx, bux,
		aval, qval;
	vector<MSKint32t> aptrb, aptre, asub, qsub;
	MSKtask_t     task = NULL;
	MSKrescodee   r;

public:
	static MSKenv_t      env;// = NULL;

	static int nDecideCalls, nHot, nInit;
	Supplier(ProblemDescription* pd, int suppN);
	Supplier();
	~Supplier();
	
	void reset(bool);
	void printSolution();

	void decide(const Prices* const, DecisionVariables* const, 
		doubleVector *flowCons = nullptr);
	//void decideQPOases(const Prices* const, DecisionVariables* const,
	//	doubleVector *flowCons = nullptr);
	void decideMosek(const Prices* const, DecisionVariables* const,
		doubleVector *flowCons = nullptr);

	void checkMatrices();

	static void MSKAPI mosekPrint(void *handle,
		MSKCONST char str[]);

};
