#pragma once
#include "Supplier.h"
#include "Network.h"

class TSO
{
	
public:
	int alternative;
	doubleVector totalFlows, temptotalFlows, prevTotalFlows, ppTotalFlows;
	TSO(ProblemDescription* pd);
	~TSO();

	Supplier *suppliers;
	double initPriceK = 0.5, step1 = 0.5, step2 = 1.5, 
		init_step = 0.01;
	int decideCalls = 0, tsoProblemCalls = 0;

	void setInitialPrices(Prices*);
	static void setInitialPrices(Prices*, double initK, ProblemDescription*);
	bool setNewCongestionPrices(DecisionVariables* decision, Prices *prices,
		double clearingSlack = 0.0001);
	bool findCapacityViolations(DecisionVariables* decisions, double capacitySlack = 0.0001);
	bool findNewCapacityViolations(DecisionVariables* decisions, double capacitySlack = 0.0001);
	bool adjustForCapacityRestrictions(DecisionVariables* decisions, Prices *prices, 
		double capacitySlack = 0.0001, double clearingSlack = 0.0001);

	bool adjustPrices(DecisionVariables* decisions, Prices *prices,
		double capacitySlack = 0.0001, double clearingSlack = 0.0001,
		double zeroSlack = 0.0001);

	bool adjustPricesTransModel(DecisionVariables* decisions, Prices *prices,
		double capacitySlack = 0.0001, double clearingSlack = 0.0001,
		double zeroSlack = 0.0001); 
	bool adjustPricesAlt2(Prices *prices,
		double capacitySlack, double clearingSlack, double dualCapSlack);
	bool adjustPricesAlt3(Prices *prices,
		double capacitySlack, double clearingSlack, double dualCapSlack);

	void reset();
	void resetStepLengths();
	void setTotalFlows(DecisionVariables* decision, bool temp = false);
	void setStepLengthFromIteration(int iteration);
	void reduceStep0(double changeFactor);

	void decide(const Prices * const prices, DecisionVariables* const decision);

	void printTotalFlows();

	int nNodes, nLinks, nSuppliers;
	doubleVector transCap, transCost1, transCost2,
		transCap_a, transCost1_a, transCost2_a,
		mcCap, supTransCap;
	double prev_sumgaps, prev_sumgaps_c,
		init_step_c = 0.005, step, step_c;
	bool printGaps = false, printCapAdjust = false,
		printSumGaps = false, transModel, tsoLinkPricing = true;
	std::vector<int> violations, prevViolations, hasOptions, hadOptions;
	doubleVector capViolDuals;
	ProblemDescription *pd;

	bool first;
	int nVariables, _nWSR = 1000, nWSR;
	double *H, *g, *lb, *ub;//, *optVal, *dualOptVal;

	vector<double> mc_a, congRent_a, newCongRent_a, prevCongRent_a, priceTarget,
		spCost, spNext;
	vector<double> linkPrices, prevLinkPrices, newLinkPrices, 
		prevOptVal, mcCap_a;
	vector<bool> hasAdjustedCong_rs;

	//Mosek
	void initTransportationSolver(MSKenv_t env);
	int numAnz, numQnz, num_a, num_paths, num_rs, numCons, numVar;
	double qCapCost = 1000;
	doubleVector c, blc, buc, blx, bux,
		aval, qval, optVal, dualOptVal;
	vector<MSKint32t> aptrb, aptre, asub, qsub;
	MSKtask_t     task = NULL;
	MSKrescodee   r;
	Network network;
	vector<bool> congestedRS, isViolated, isOnCap_rs, isOnCap_a;
	vector<double> transDemand, mc_rs;
	double solverSlack = 1E-2, maxDual;
	bool changedPrice;
	void initMatrixes();
	void runMosek();
	void increasePrice(Prices *prices, int link, double gap);
	void decreasePrice(Prices *prices, int link, double gap);
	void bringPriceCloserToMC(Prices *prices, int link, double gap);

	void findShortestPaths();
	void printCapacityViolations();
	void initQPSolver();
	bool checkCapViolations(DecisionVariables* decisions, Prices *prices,
		double capacitySlack, double dualSlack);
};

class TabuTSO : public TSO
{

public:

	TabuTSO(ProblemDescription *pd, int listLength);
	~TabuTSO();

	bool setNewCongestionPrices(
		DecisionVariables* decision,
		Prices *prices,
		double clearingSlack = 0.0001
		);

	bool adjustPrices(
		DecisionVariables* decisions,
		Prices *prices,
		double capacitySlack = 0.0001,
		double clearingSlack = 0.0001,
		double zeroSlack = 0.0001
		);

protected:
	vector<vector<double>> linkPriceHistory;
	int
		tabuListLength,
		newestElement; // Indexes the link price history to create an efficient FIFO queue (compared to pushing at the front of a vector and removing from the back)
	bool approveSuggestion(vector<double> suggestion);


private:
	void addToHistory(vector<double> prices);

};