*********************
* Valg av datasett:
$set dataset "set01_simple-supplier";
*$set dataset "set02_cap-split";
*$set dataset "set03_cost-split";
*$set dataset "set04_5-node-congested";
*$set dataset "set05_5-node-uncongested";
*$set dataset "set06_8-node-congested";
*$set dataset "set06b_8-node-multiprod";
*$set dataset "set07_8-node-uncongested";
*$set dataset "set08_13-node-congested";
*$set dataset "set09_13-node-uncongested";
*---------------
* Bruk den nederste for  kjre kommandolinje-GAMS, og den verste for GUI-gams
$include "data\gms\datasets\%dataset%.INC"
*$include "data\gms\datasets\%gams.u1%.INC"
*---------------
alias(i,j);

positive variables
         s(n,i)  sales by producer i in node n
         q(n,i)  production by producer i in node n
         f(n,m,i)  flow directly from node n to node m belongning to producer i
         g(n,m)  total flow directly from node n to node m
         beta1(n,i)  dual of the producer capacity constraint (prod_capacity)
         beta3(n,m)  dual of the tso capacity constraint (tflow_capacity)
;

variables
         pi(n)  the final commodity price in node (market) n
         alpha(n,m)  the unit transportation price on the link from node n to node m and also the dual of market_clearing2
         beta2(n,i)  dual of the sales equality constraint (sales_lagrange)
;

equations
         sales_lagrange(n,i) enforces producer i sales in node n equals production + flow in - flow out
         production_lagrange(n,i)  producer marginal costs
         flow_lagrange(n,m,i)  lagrangian of the tso
         prod_capacity(n,i)  enforces producer production limits
         flow_conservation(n,i)  conserves flow production and sales in each node
         tflow_lagrange(n,m)
         tflow_capacity(n,m)  enforces total link flows below or at capacity
         market_clearing1(n)  enforces the price-quantity relationship in all markets
         market_clearing2(n,m)  ensures the total flow on link (n->m) equals all the suppliers flows
;

sales_lagrange(n,i)..            ( - 1)*(B(n) - A(n)*sum(j,s(n,j))) + A(n)*s(n,i) + beta2(n,i) =g= 0;
*Market power added (A(n)*s(n,i))
*dual from mcc NOT added, A(n)*pi(n)
production_lagrange(n,i)..       Cprod1(i) + 2*Cprod2(i)*q(n,i) + beta1(n,i) - beta2(n,i) =g= 0;
flow_lagrange(n,m,i)..           alpha(n,m) + beta2(n,i) - beta2(m,i) =g= 0;
*dual from mcc NOT added, alpha(n,m)
prod_capacity(n,i)..             productionCAP(n,i) - q(n,i) =g= 0;
flow_conservation(n,i)..         s(n,i) - q(n,i) + sum(m, f(n,m,i) - f(m,n,i)) =e= 0;
tflow_lagrange(n,m)..             - alpha(n,m) + Ctrans1(n,m) + 2*Ctrans2(n,m)*g(n,m) + beta3(n,m) =g= 0;
*dual from mcc NOT added, - alpha(n,m)
tflow_capacity(n,m)..            transportationCAP(n,m) - g(n,m) =g= 0;
market_clearing1(n)..            pi(n) - B(n) + A(n)*sum(i,s(n,i)) =e= 0;
market_clearing2(n,m)..          -sum(i,f(n,m,i)) + g(n,m) =e= 0;

model eq /sales_lagrange.s, production_lagrange.q, flow_lagrange.f,
         prod_capacity.beta1, flow_conservation.beta2, tflow_lagrange.g,
         tflow_capacity.beta3, market_clearing2.alpha, market_clearing1.pi / ;


scalar starttime; starttime = jnow;

*pi.lo('1') = 8.7138;
*pi.up('1') = 8.71389;

*pi.lo('1') = 8.6873;
*pi.up('1') = 8.68739;


* Set use the options file for path
*eq.optfile = 1;
* These may be needed
*option iterlim = 200000;
*eq.iterlim = 200000;

solve eq using mcp;


scalar elapsed; elapsed = (jnow - starttime)*24*3600;
display elapsed;

Scalar tcomp, texec, telapsed;
tcomp = TimeComp;
texec = TimeExec;
telapsed = TimeElapsed;
Display tcomp, texec, telapsed;


* OUTPUT
file res / data\gms\results\%dataset%.csv /;
res.pc = 5;

put res ;

*Info
put "Nodes",card(n)/
put "Producers",card(i)/

put "Node","Price"/
loop((n),
         put n.tl,(B(n) - A(n)*sum(j,s.l(n,j)))/
);

put "Node","Producer","Production","Sales"/
loop((i,n),
         put n.tl,i.tl,q.l(n,i),s.l(n,i)/
);

put "From node","To node","Tarif","Total flow"/
loop((n,m),
         put n.tl,m.tl,alpha.l(n,m),g.l(n,m)/
);
put "From node", "To node", "Producer","Producer flow"/
loop((n,m,i),
         put n.tl,m.tl,i.tl,f.l(n,m,i)/
);
putclose;

